@@ -5,15 +5,15 @@
  *
  * (c) Dirk Hohndel 2013
  */
-#include "ssrf.h"
 #include <assert.h>
 #include <unistd.h>
 #include <ctype.h>
 #include <string.h>
-#include "dive.h"
+#include "ssrf.h"
+#include "divelist.h"
 #include "subsurface-string.h"
 #include "deco.h"
-#include "divelist.h"
+#include "errorhelper.h"
 #include "planner.h"
 #include "gettext.h"
 #include "libdivecomputer/parser.h"
@@ -36,7 +36,6 @@ static int decostoplevels_imperial[] = { 0, 3048, 6096, 9144, 12192, 15240, 1828
 					325120, 345440, 365760, 386080 };
 
 char *disclaimer;
-int plot_depth = 0;
 #if DEBUG_PLAN
 void dump_plan(struct diveplan *diveplan)
 {
@@ -94,7 +93,7 @@ int get_gasidx(struct dive *dive, struct gasmix mix)
 	return find_best_gasmix_match(mix, dive->cylinder);
 }
 
-void interpolate_transition(struct deco_state *ds, struct dive *dive, duration_t t0, duration_t t1, depth_t d0, depth_t d1, struct gasmix gasmix, o2pressure_t po2, enum divemode_t divemode)
+static void interpolate_transition(struct deco_state *ds, struct dive *dive, duration_t t0, duration_t t1, depth_t d0, depth_t d1, struct gasmix gasmix, o2pressure_t po2, enum divemode_t divemode)
 {
 	int32_t j;
 
@@ -103,11 +102,11 @@ void interpolate_transition(struct deco_state *ds, struct dive *dive, duration_t
 		add_segment(ds, depth_to_bar(depth, dive), gasmix, 1, po2.mbar, divemode, prefs.bottomsac);
 	}
 	if (d1.mm > d0.mm)
-		calc_crushing_pressure(ds, depth_to_bar(d1.mm, &displayed_dive));
+		calc_crushing_pressure(ds, depth_to_bar(d1.mm, dive));
 }
 
 /* returns the tissue tolerance at the end of this (partial) dive */
-int tissue_at_end(struct deco_state *ds, struct dive *dive, struct deco_state **cached_datap)
+static int tissue_at_end(struct deco_state *ds, struct dive *dive, struct deco_state **cached_datap)
 {
 	struct divecomputer *dc;
 	struct sample *sample, *psample;
@@ -179,9 +178,10 @@ int tissue_at_end(struct deco_state *ds, struct dive *dive, struct deco_state **
 
 
 /* if a default cylinder is set, use that */
-void fill_default_cylinder(cylinder_t *cyl)
+void fill_default_cylinder(struct dive *dive, int idx)
 {
 	const char *cyl_name = prefs.default_cylinder;
+	cylinder_t *cyl = &dive->cylinder[idx];
 	struct tank_info_t *ti = tank_info;
 	pressure_t pO2 = {.mbar = 1600};
 
@@ -205,7 +205,7 @@ void fill_default_cylinder(cylinder_t *cyl)
 			cyl->type.size.mliter = lrint(cuft_to_l(ti->cuft) * 1000 / bar_to_atm(psi_to_bar(ti->psi)));
 	}
 	// MOD of air
-	cyl->depth = gas_mod(cyl->gasmix, pO2, &displayed_dive, 1);
+	cyl->depth = gas_mod(cyl->gasmix, pO2, dive, 1);
 }
 
 /* calculate the new end pressure of the cylinder, based on its current end pressure and the
@@ -233,7 +233,7 @@ static void update_cylinder_pressure(struct dive *d, int old_depth, int new_dept
 	}
 }
 
-/* simply overwrite the data in the displayed_dive
+/* overwrite the data in dive
  * return false if something goes wrong */
 static void create_dive_from_plan(struct diveplan *diveplan, struct dive *dive, bool track_gas)
 {
@@ -256,7 +256,7 @@ static void create_dive_from_plan(struct diveplan *diveplan, struct dive *dive,
 #endif
 	dive->salinity = diveplan->salinity;
 	// reset the cylinders and clear out the samples and events of the
-	// displayed dive so we can restart
+	// dive-to-be-planned so we can restart
 	reset_cylinders(dive, track_gas);
 	dc = &dive->dc;
 	dc->when = dive->when = diveplan->when;
@@ -343,7 +343,7 @@ static void create_dive_from_plan(struct diveplan *diveplan, struct dive *dive,
 	dive->dc.last_manual_time.seconds = last_manual_point;
 
 #if DEBUG_PLAN & 32
-	save_dive(stdout, &displayed_dive);
+	save_dive(stdout, dive);
 #endif
 	return;
 }
@@ -376,7 +376,7 @@ struct divedatapoint *create_dp(int time_incr, int depth, int cylinderid, int po
 	return dp;
 }
 
-void add_to_end_of_diveplan(struct diveplan *diveplan, struct divedatapoint *dp)
+static void add_to_end_of_diveplan(struct diveplan *diveplan, struct divedatapoint *dp)
 {
 	struct divedatapoint **lastdp = &diveplan->dp;
 	struct divedatapoint *ldp = *lastdp;
@@ -406,13 +406,12 @@ struct gaschanges {
 	int gasidx;
 };
 
-
-static struct gaschanges *analyze_gaslist(struct diveplan *diveplan, int *gaschangenr, int depth, int *asc_cylinder)
+static struct gaschanges *analyze_gaslist(struct diveplan *diveplan, struct dive *dive, int *gaschangenr, int depth, int *asc_cylinder)
 {
 	int nr = 0;
 	struct gaschanges *gaschanges = NULL;
 	struct divedatapoint *dp = diveplan->dp;
-	int best_depth = displayed_dive.cylinder[*asc_cylinder].depth.mm;
+	int best_depth = dive->cylinder[*asc_cylinder].depth.mm;
 	bool total_time_zero = true;
 	while (dp) {
 		if (dp->time == 0 && total_time_zero) {
@@ -447,7 +446,7 @@ static struct gaschanges *analyze_gaslist(struct diveplan *diveplan, int *gascha
 	for (nr = 0; nr < *gaschangenr; nr++) {
 		int idx = gaschanges[nr].gasidx;
 		printf("gaschange nr %d: @ %5.2lfm gasidx %d (%s)\n", nr, gaschanges[nr].depth / 1000.0,
-		       idx, gasname(&displayed_dive.cylinder[idx].gasmix));
+		       idx, gasname(dive->cylinder[idx].gasmix));
 	}
 #endif
 	return gaschanges;
@@ -528,15 +527,16 @@ int ascent_velocity(int depth, int avg_depth, int bottom_time)
 	}
 }
 
-void track_ascent_gas(int depth, cylinder_t *cylinder, int avg_depth, int bottom_time, bool safety_stop, enum divemode_t divemode)
+static void track_ascent_gas(int depth, struct dive *dive, int cylinder_id, int avg_depth, int bottom_time, bool safety_stop, enum divemode_t divemode)
 {
+	cylinder_t *cylinder = &dive->cylinder[cylinder_id];
 	while (depth > 0) {
 		int deltad = ascent_velocity(depth, avg_depth, bottom_time) * TIMESTEP;
 		if (deltad > depth)
 			deltad = depth;
-		update_cylinder_pressure(&displayed_dive, depth, depth - deltad, TIMESTEP, prefs.decosac, cylinder, true, divemode);
+		update_cylinder_pressure(dive, depth, depth - deltad, TIMESTEP, prefs.decosac, cylinder, true, divemode);
 		if (depth <= 5000 && depth >= (5000 - deltad) && safety_stop) {
-			update_cylinder_pressure(&displayed_dive, 5000, 5000, 180, prefs.decosac, cylinder, true, divemode);
+			update_cylinder_pressure(dive, 5000, 5000, 180, prefs.decosac, cylinder, true, divemode);
 			safety_stop = false;
 		}
 		depth -= deltad;
@@ -544,7 +544,7 @@ void track_ascent_gas(int depth, cylinder_t *cylinder, int avg_depth, int bottom
 }
 
 // Determine whether ascending to the next stop will break the ceiling.  Return true if the ascent is ok, false if it isn't.
-bool trial_ascent(struct deco_state *ds, int wait_time, int trial_depth, int stoplevel, int avg_depth, int bottom_time, struct gasmix gasmix, int po2, double surface_pressure, struct dive *dive, enum divemode_t divemode)
+static bool trial_ascent(struct deco_state *ds, int wait_time, int trial_depth, int stoplevel, int avg_depth, int bottom_time, struct gasmix gasmix, int po2, double surface_pressure, struct dive *dive, enum divemode_t divemode)
 {
 
 	bool clear_to_ascend = true;
@@ -558,23 +558,28 @@ bool trial_ascent(struct deco_state *ds, int wait_time, int trial_depth, int sto
 		add_segment(ds, depth_to_bar(trial_depth, dive),
 			    gasmix,
 			    wait_time, po2, divemode, prefs.decosac);
-	if (decoMode() == VPMB && (deco_allowed_depth(tissue_tolerance_calc(ds, dive,depth_to_bar(stoplevel, dive)),
-						      surface_pressure, dive, 1)
-				   > stoplevel)) {
-		restore_deco_state(trial_cache, ds, false);
-		free(trial_cache);
-		return false;
+	if (decoMode() == VPMB) {
+		double tolerance_limit = tissue_tolerance_calc(ds, dive, depth_to_bar(stoplevel, dive));
+		update_regression(ds, dive);
+		if (deco_allowed_depth(tolerance_limit, surface_pressure, dive, 1) > stoplevel) {
+			restore_deco_state(trial_cache, ds, false);
+			free(trial_cache);
+			return false;
+		}
 	}
 
 	while (trial_depth > stoplevel) {
+		double tolerance_limit;
 		int deltad = ascent_velocity(trial_depth, avg_depth, bottom_time) * TIMESTEP;
 		if (deltad > trial_depth) /* don't test against depth above surface */
 			deltad = trial_depth;
 		add_segment(ds, depth_to_bar(trial_depth, dive),
 			    gasmix,
 			    TIMESTEP, po2, divemode, prefs.decosac);
-		if (deco_allowed_depth(tissue_tolerance_calc(ds, dive, depth_to_bar(trial_depth, dive)),
-				       surface_pressure, dive, 1) > trial_depth - deltad) {
+		tolerance_limit = tissue_tolerance_calc(ds, dive, depth_to_bar(trial_depth, dive));
+		if (decoMode() == VPMB)
+			update_regression(ds, dive);
+		if (deco_allowed_depth(tolerance_limit, surface_pressure, dive, 1) > trial_depth - deltad) {
 			/* We should have stopped */
 			clear_to_ascend = false;
 			break;
@@ -590,10 +595,9 @@ bool trial_ascent(struct deco_state *ds, int wait_time, int trial_depth, int sto
  * Also return true if this cannot be calculated because the cylinder doesn't have
  * size or a starting pressure.
  */
-bool enough_gas(int current_cylinder)
+static bool enough_gas(const struct dive *dive, int current_cylinder)
 {
-	cylinder_t *cyl;
-	cyl = &displayed_dive.cylinder[current_cylinder];
+	const cylinder_t *cyl = &dive->cylinder[current_cylinder];
 
 	if (!cyl->start.mbar)
 		return true;
@@ -608,8 +612,7 @@ bool enough_gas(int current_cylinder)
  * leap is a guess for the maximum but there is no guarantee that leap is an upper limit.
  * So we always test at the upper bundary, not in the middle!
  */
-
-int wait_until(struct deco_state *ds, struct dive *dive, int clock, int min, int leap, int stepsize, int depth, int target_depth, int avg_depth, int bottom_time, struct gasmix gasmix, int po2, double surface_pressure, enum divemode_t divemode)
+static int wait_until(struct deco_state *ds, struct dive *dive, int clock, int min, int leap, int stepsize, int depth, int target_depth, int avg_depth, int bottom_time, struct gasmix gasmix, int po2, double surface_pressure, enum divemode_t divemode)
 {
 	// When a deco stop exceeds two days, there is something wrong...
 	if (min >= 48 * 3600)
@@ -626,15 +629,30 @@ int wait_until(struct deco_state *ds, struct dive *dive, int clock, int min, int
 	return wait_until(ds, dive, clock, min, leap / 2, stepsize, depth, target_depth, avg_depth, bottom_time, gasmix, po2, surface_pressure, divemode);
 }
 
-// Work out the stops. Return value is if there were any mandatory stops.
-
-
-void printdecotable(struct decostop *table)
+static void average_max_depth(struct diveplan *dive, int *avg_depth, int *max_depth)
 {
-	while (table->depth) {
-		printf("depth=%d time=%d\n", table->depth, table->time);
-		++table;
+	int integral = 0;
+	int last_time = 0;
+	int last_depth = 0;
+	struct divedatapoint *dp = dive->dp;
+
+	*max_depth = 0;
+
+	while (dp) {
+		if (dp->time) {
+			/* Ignore gas indication samples */
+			integral += (dp->depth.mm + last_depth) * (dp->time - last_time) / 2;
+			last_time = dp->time;
+			last_depth = dp->depth.mm;
+			if (dp->depth.mm > *max_depth)
+				*max_depth = dp->depth.mm;
+		}
+		dp = dp->next;
 	}
+	if (last_time)
+		*avg_depth = integral / last_time;
+	else
+		*avg_depth = *max_depth = 0;
 }
 
 bool plan(struct deco_state *ds, struct diveplan *diveplan, struct dive *dive, int timestep, struct decostop *decostoptable, struct deco_state **cached_datap, bool is_planner, bool show_disclaimer)
@@ -744,7 +762,7 @@ bool plan(struct deco_state *ds, struct diveplan *diveplan, struct dive *dive, i
 		gaschanges = NULL;
 		gaschangenr = 0;
 	} else {
-		gaschanges = analyze_gaslist(diveplan, &gaschangenr, depth, &best_first_ascend_cylinder);
+		gaschanges = analyze_gaslist(diveplan, dive, &gaschangenr, depth, &best_first_ascend_cylinder);
 	}
 	/* Find the first potential decostopdepth above current depth */
 	for (stopidx = 0; stopidx < decostoplevelcount; stopidx++)
@@ -764,7 +782,7 @@ bool plan(struct deco_state *ds, struct diveplan *diveplan, struct dive *dive, i
 	vpmb_start_gradient(ds);
 	if (decoMode() == RECREATIONAL) {
 		bool safety_stop = prefs.safetystop && max_depth >= 10000;
-		track_ascent_gas(depth, &dive->cylinder[current_cylinder], avg_depth, bottom_time, safety_stop, divemode);
+		track_ascent_gas(depth, dive, current_cylinder, avg_depth, bottom_time, safety_stop, divemode);
 		// How long can we stay at the current depth and still directly ascent to the surface?
 		do {
 			add_segment(ds, depth_to_bar(depth, dive),
@@ -774,7 +792,7 @@ bool plan(struct deco_state *ds, struct diveplan *diveplan, struct dive *dive, i
 			clock += timestep;
 		} while (trial_ascent(ds, 0, depth, 0, avg_depth, bottom_time, dive->cylinder[current_cylinder].gasmix,
 				      po2, diveplan->surface_pressure / 1000.0, dive, divemode) &&
-			 enough_gas(current_cylinder) && clock < 6 * 3600);
+			 enough_gas(dive, current_cylinder) && clock < 6 * 3600);
 
 		// We did stay one DECOTIMESTEP too many.
 		// In the best of all worlds, we would roll back also the last add_segment in terms of caching deco state, but
@@ -865,7 +883,8 @@ bool plan(struct deco_state *ds, struct diveplan *diveplan, struct dive *dive, i
 		first_stop_depth = 0;
 		stopidx = bottom_stopidx;
 		ds->first_ceiling_pressure.mbar = depth_to_mbar(
-					deco_allowed_depth(tissue_tolerance_calc(ds, dive, depth_to_bar(depth, dive)), diveplan->surface_pressure / 1000.0, dive, 1),
+					deco_allowed_depth(tissue_tolerance_calc(ds, dive, depth_to_bar(depth, dive)),
+							   diveplan->surface_pressure / 1000.0, dive, 1),
 					dive);
 		if (ds->max_bottom_ceiling_pressure.mbar > ds->first_ceiling_pressure.mbar)
 			ds->first_ceiling_pressure.mbar = ds->max_bottom_ceiling_pressure.mbar;
@@ -881,7 +900,7 @@ bool plan(struct deco_state *ds, struct diveplan *diveplan, struct dive *dive, i
 			report_error(translate("gettextFromC", "Can't find gas %s"), gasname(gas));
 			current_cylinder = 0;
 		}
-		reset_regression();
+		reset_regression(ds);
 		while (1) {
 			/* We will break out when we hit the surface */
 			do {
@@ -905,7 +924,7 @@ bool plan(struct deco_state *ds, struct diveplan *diveplan, struct dive *dive, i
 				depth -= deltad;
 				/* Print VPM-Gradient as gradient factor, this has to be done from within deco.c */
 				if (decodive)
-					plot_depth = depth;
+					ds->plot_depth = depth;
 			} while (depth > 0 && depth > stoplevels[stopidx]);
 
 			if (depth <= 0)
@@ -1076,8 +1095,8 @@ bool plan(struct deco_state *ds, struct diveplan *diveplan, struct dive *dive, i
 
 	plan_add_segment(diveplan, clock - previous_point_time, 0, current_cylinder, po2, false, divemode);
 	if (decoMode() == VPMB) {
-		diveplan->eff_gfhigh = lrint(100.0 * regressionb());
-		diveplan->eff_gflow = lrint(100.0 * (regressiona() * first_stop_depth + regressionb()));
+		diveplan->eff_gfhigh = lrint(100.0 * regressionb(ds));
+		diveplan->eff_gflow = lrint(100.0 * (regressiona(ds) * first_stop_depth + regressionb(ds)));
 	}
 
 	for (int i = 0; i < MAX_CYLINDERS; i++)
