@@ -19,7 +19,6 @@ extern "C" {
 #define N2_DENSITY 1165
 #define HE_DENSITY 166
 #define SURFACE_PRESSURE 1013 // mbar
-#define SURFACE_PRESSURE_STRING "1013"
 #define ZERO_C_IN_MKELVIN 273150 // mKelvin
 
 #ifdef __cplusplus
@@ -106,12 +105,12 @@ typedef struct
 
 typedef struct
 {
-	uint32_t mkelvin; // up to 4 MdegK (temperatures in K are always positive)
+	uint32_t mkelvin; // up to 4 MK (temperatures in K are always positive)
 } temperature_t;
 
 typedef struct
 {
-	uint64_t mkelvin; // up to 18446744073 MdegK (temperatures in K are always positive)
+	uint64_t mkelvin; // up to 18446744073 MK (temperatures in K are always positive)
 } temperature_sum_t;
 
 typedef struct
@@ -138,6 +137,8 @@ typedef struct pos {
 	degrees_t lat, lon;
 } location_t;
 
+extern void parse_location(const char *, location_t *);
+
 static inline bool has_location(const location_t *loc)
 {
 	return loc->lat.udeg || loc->lon.udeg;
@@ -253,6 +254,17 @@ static inline int mbar_to_PSI(int mbar)
 	return to_PSI(p);
 }
 
+static inline int32_t altitude_to_pressure(int32_t altitude) 	// altitude in mm above sea level
+{						// returns atmospheric pressure in mbar
+	return (int32_t) (1013.0 * exp(- altitude / 7800000.0));
+}
+
+
+static inline int32_t pressure_to_altitude(int32_t pressure)	// pressure in mbar
+{						// returns altitude in mm above sea level
+	return (int32_t) (log(1013.0 / pressure) * 7800000);
+}
+
 /*
  * We keep our internal data in well-specified units, but
  * the input and output may come in some random format. This
@@ -307,15 +319,28 @@ struct units {
 #define SI_UNITS                                                                                           \
         {                                                                                                  \
 	        .length = METERS, .volume = LITER, .pressure = BAR, .temperature = CELSIUS, .weight = KG,  \
-		.vertical_speed_time = MINUTES, .duration_units = MIXED, .show_units_table = false              \
+		.vertical_speed_time = MINUTES, .duration_units = MIXED, .show_units_table = false         \
         }
 
 #define IMPERIAL_UNITS                                                                                     \
         {                                                                                                  \
 	        .length = FEET, .volume = CUFT, .pressure = PSI, .temperature = FAHRENHEIT, .weight = LBS, \
-		.vertical_speed_time = MINUTES, .duration_units = MIXED, .show_units_table = false              \
+		.vertical_speed_time = MINUTES, .duration_units = MIXED, .show_units_table = false         \
         }
 
+extern const struct units SI_units, IMPERIAL_units;
+
+extern const struct units *get_units(void);
+
+extern int get_pressure_units(int mb, const char **units);
+extern double get_depth_units(int mm, int *frac, const char **units);
+extern double get_volume_units(unsigned int ml, int *frac, const char **units);
+extern double get_temp_units(unsigned int mk, const char **units);
+extern double get_weight_units(unsigned int grams, int *frac, const char **units);
+extern double get_vertical_speed_units(unsigned int mms, int *frac, const char **units);
+
+extern depth_t units_to_depth(double depth);
+extern int units_to_sac(double volume);
 #ifdef __cplusplus
 }
 #endif
