@@ -8,6 +8,10 @@
 #include "qt-models/filtermodels.h"
 #include "core/divesitehelpers.h"
 #include "desktop-widgets/modeldelegates.h"
+#include "core/subsurface-qt/DiveListNotifier.h"
+#include "command.h"
+#include "core/taxonomy.h"
+#include "core/settings/qPrefUnit.h"
 
 #include <QDebug>
 #include <QShowEvent>
@@ -17,35 +21,34 @@
 #include <QDesktopWidget>
 #include <QScrollBar>
 
-LocationInformationWidget::LocationInformationWidget(QWidget *parent) : QGroupBox(parent), modified(false), diveSite(nullptr)
+LocationInformationWidget::LocationInformationWidget(QWidget *parent) : QGroupBox(parent), diveSite(nullptr), closeDistance(0)
 {
-	memset(&taxonomy, 0, sizeof(taxonomy));
 	ui.setupUi(this);
 	ui.diveSiteMessage->setCloseButtonVisible(false);
 
-	acceptAction = new QAction(tr("Apply changes"), this);
-	connect(acceptAction, SIGNAL(triggered(bool)), this, SLOT(acceptChanges()));
-
-	rejectAction = new QAction(tr("Discard changes"), this);
-	connect(rejectAction, SIGNAL(triggered(bool)), this, SLOT(rejectChanges()));
+	QAction *acceptAction = new QAction(tr("Done"), this);
+	connect(acceptAction, &QAction::triggered, this, &LocationInformationWidget::acceptChanges);
 
 	ui.diveSiteMessage->setText(tr("Dive site management"));
 	ui.diveSiteMessage->addAction(acceptAction);
-	ui.diveSiteMessage->addAction(rejectAction);
 
 	connect(ui.geoCodeButton, SIGNAL(clicked()), this, SLOT(reverseGeocode()));
-	connect(this, SIGNAL(nameChanged(const QString &, const QString &)),
-		LocationFilterModel::instance(), SLOT(changeName(const QString &, const QString &)));
-	connect(ui.updateLocationButton, SIGNAL(clicked()), this, SLOT(updateLocationOnMap()));
-	connect(ui.diveSiteCoordinates, SIGNAL(returnPressed()), this, SLOT(updateLocationOnMap()));
 	ui.diveSiteCoordinates->installEventFilter(this);
 
+	connect(&diveListNotifier, &DiveListNotifier::diveSiteChanged, this, &LocationInformationWidget::diveSiteChanged);
+	connect(qPrefUnits::instance(), &qPrefUnits::unit_systemChanged, this, &LocationInformationWidget::unitsChanged);
+	unitsChanged();
+
 	ui.diveSiteListView->setModel(&filter_model);
 	ui.diveSiteListView->setModelColumn(LocationInformationModel::NAME);
 	ui.diveSiteListView->installEventFilter(this);
-	// Map Management Code.
-	connect(MapWidget::instance(), &MapWidget::coordinatesChanged,
-		this, &LocationInformationWidget::updateGpsCoordinates);
+}
+
+void LocationInformationWidget::keyPressEvent(QKeyEvent *e)
+{
+	if (e->key() == Qt::Key_Escape)
+		MainWindow::instance()->setFocus();
+	return QGroupBox::keyPressEvent(e);
 }
 
 bool LocationInformationWidget::eventFilter(QObject *object, QEvent *ev)
@@ -56,8 +59,6 @@ bool LocationInformationWidget::eventFilter(QObject *object, QEvent *ev)
 		contextMenu.addAction(tr("Merge into current site"), this, SLOT(mergeSelectedDiveSites()));
 		contextMenu.exec(ctx->globalPos());
 		return true;
-	} else if (ev->type() == QEvent::FocusOut && object == ui.diveSiteCoordinates) {
-		updateLocationOnMap();
 	}
 	return false;
 }
@@ -65,31 +66,22 @@ bool LocationInformationWidget::eventFilter(QObject *object, QEvent *ev)
 void LocationInformationWidget::enableLocationButtons(bool enable)
 {
 	ui.geoCodeButton->setEnabled(enable);
-	ui.updateLocationButton->setEnabled(enable);
 }
 
 void LocationInformationWidget::mergeSelectedDiveSites()
 {
 	if (!diveSite)
 		return;
-	if (QMessageBox::warning(MainWindow::instance(), tr("Merging dive sites"),
-				 tr("You are about to merge dive sites, you can't undo that action \n Are you sure you want to continue?"),
-				 QMessageBox::Ok, QMessageBox::Cancel) != QMessageBox::Ok)
-		return;
 
-	QModelIndexList selection = ui.diveSiteListView->selectionModel()->selectedIndexes();
-	// std::vector guarantees contiguous storage and can therefore be passed to C-code
-	std::vector<struct dive_site *> selected_dive_sites;
+	const QModelIndexList selection = ui.diveSiteListView->selectionModel()->selectedIndexes();
+	QVector<dive_site *> selected_dive_sites;
 	selected_dive_sites.reserve(selection.count());
-	Q_FOREACH (const QModelIndex &idx, selection) {
+	for (const QModelIndex &idx: selection) {
 		dive_site *ds = idx.data(LocationInformationModel::DIVESITE_ROLE).value<dive_site *>();
 		if (ds)
 			selected_dive_sites.push_back(ds);
 	}
-	merge_dive_sites(diveSite, selected_dive_sites.data(), (int)selected_dive_sites.size());
-	LocationInformationModel::instance()->update();
-	QSortFilterProxyModel *m = (QSortFilterProxyModel *)ui.diveSiteListView->model();
-	m->invalidate();
+	Command::mergeDiveSites(diveSite, selected_dive_sites);
 }
 
 void LocationInformationWidget::updateLabels()
@@ -102,7 +94,7 @@ void LocationInformationWidget::updateLabels()
 		ui.diveSiteName->setText(diveSite->name);
 	else
 		ui.diveSiteName->clear();
-	const char *country = taxonomy_get_country(&taxonomy);
+	const char *country = taxonomy_get_country(&diveSite->taxonomy);
 	if (country)
 		ui.diveSiteCountry->setText(country);
 	else
@@ -115,15 +107,57 @@ void LocationInformationWidget::updateLabels()
 		ui.diveSiteNotes->setPlainText(diveSite->notes);
 	else
 		ui.diveSiteNotes->clear();
-	if (has_location(&diveSite->location)) {
-		const char *coords = printGPSCoords(&diveSite->location);
-		ui.diveSiteCoordinates->setText(coords);
-		free((void *)coords);
-	} else {
+	if (has_location(&diveSite->location))
+		ui.diveSiteCoordinates->setText(printGPSCoords(&diveSite->location));
+	else
 		ui.diveSiteCoordinates->clear();
-	}
 
-	ui.locationTags->setText(constructLocationTags(&taxonomy, false));
+	ui.locationTags->setText(constructLocationTags(&diveSite->taxonomy, false));
+}
+
+void LocationInformationWidget::unitsChanged()
+{
+	if (prefs.units.length == units::METERS) {
+		ui.diveSiteDistanceUnits->setText("m");
+		ui.diveSiteDistance->setText(QString::number(lrint(closeDistance / 1000.0)));
+	} else {
+		ui.diveSiteDistanceUnits->setText("ft");
+		ui.diveSiteDistance->setText(QString::number(lrint(mm_to_feet(closeDistance))));
+	}
+}
+
+void LocationInformationWidget::diveSiteChanged(struct dive_site *ds, int field)
+{
+	if (diveSite != ds)
+		return; // A different dive site was changed -> do nothing.
+	switch (field) {
+	case LocationInformationModel::NAME:
+		ui.diveSiteName->setText(diveSite->name);
+		MapWidget::instance()->repopulateLabels();
+		return;
+	case LocationInformationModel::DESCRIPTION:
+		ui.diveSiteDescription->setText(diveSite->description);
+		return;
+	case LocationInformationModel::NOTES:
+		ui.diveSiteNotes->setText(diveSite->notes);
+		return;
+	case LocationInformationModel::TAXONOMY:
+		ui.diveSiteCountry->setText(taxonomy_get_country(&diveSite->taxonomy));
+		ui.locationTags->setText(constructLocationTags(&diveSite->taxonomy, false));
+		return;
+	case LocationInformationModel::LOCATION:
+		filter_model.setCoordinates(diveSite->location);
+		if (has_location(&diveSite->location)) {
+			enableLocationButtons(true);
+			ui.diveSiteCoordinates->setText(printGPSCoords(&diveSite->location));
+		} else {
+			enableLocationButtons(false);
+			ui.diveSiteCoordinates->clear();
+		}
+		MapWidget::instance()->repopulateLabels();
+	default:
+		return;
+	}
 }
 
 void LocationInformationWidget::clearLabels()
@@ -136,234 +170,137 @@ void LocationInformationWidget::clearLabels()
 	ui.locationTags->clear();
 }
 
-void LocationInformationWidget::updateGpsCoordinates(const location_t &location)
-{
-	QString oldText = ui.diveSiteCoordinates->text();
-
-	const char *coords = printGPSCoords(&location);
-	ui.diveSiteCoordinates->setText(coords);
-	enableLocationButtons(has_location(&location));
-	free((void *)coords);
-	if (oldText != ui.diveSiteCoordinates->text())
-		markChangedWidget(ui.diveSiteCoordinates);
-}
-
-// Parse GPS text into latitude and longitude.
-// On error, false is returned and the output parameters are left unmodified.
-bool parseGpsText(const QString &text, location_t &location)
+// Parse GPS text into location_t
+static location_t parseGpsText(const QString &text)
 {
 	double lat, lon;
-	if (parseGpsText(text, &lat, &lon)) {
-		location = create_location(lat, lon);
-		return true;
-	}
-	return false;
+	if (parseGpsText(text, &lat, &lon))
+		return create_location(lat, lon);
+	return { {0}, {0} };
 }
 
 void LocationInformationWidget::acceptChanges()
 {
-	if (!diveSite) {
-		qWarning() << "did not have valid dive site in LocationInformationWidget";
-		return;
-	}
-
-	char *uiString;
-	uiString = copy_qstring(ui.diveSiteName->text());
-	if (!same_string(uiString, diveSite->name)) {
-		emit nameChanged(QString(diveSite->name), ui.diveSiteName->text());
-		free(diveSite->name);
-		diveSite->name = uiString;
-	} else {
-		free(uiString);
-	}
-	uiString = copy_qstring(ui.diveSiteDescription->text());
-	if (!same_string(uiString, diveSite->description)) {
-		free(diveSite->description);
-		diveSite->description = uiString;
-	} else {
-		free(uiString);
-	}
-	uiString = copy_qstring(ui.diveSiteCountry->text());
-	// if the user entered a different country, first update the local taxonomy
-	// this below will get copied into the diveSite
-	if (!same_string(uiString, taxonomy_get_country(&taxonomy)) &&
-	    !empty_string(uiString))
-		taxonomy_set_country(&taxonomy, uiString, taxonomy_origin::GEOMANUAL);
-	else
-		free(uiString);
-	// now update the diveSite
-	copy_taxonomy(&taxonomy, &diveSite->taxonomy);
-
-	uiString = copy_qstring(ui.diveSiteNotes->document()->toPlainText());
-	if (!same_string(uiString, diveSite->notes)) {
-		free(diveSite->notes);
-		diveSite->notes = uiString;
-	} else {
-		free(uiString);
-	}
-
-	if (!ui.diveSiteCoordinates->text().isEmpty())
-		parseGpsText(ui.diveSiteCoordinates->text(), diveSite->location);
-	if (dive_site_is_empty(diveSite)) {
-		LocationInformationModel::instance()->removeRow(get_divesite_idx(diveSite));
-		displayed_dive.dive_site = nullptr;
-		diveSite = nullptr;
-	}
-	mark_divelist_changed(true);
-	resetState();
-}
-
-void LocationInformationWidget::rejectChanges()
-{
-	resetState();
+	MainWindow::instance()->diveList->setEnabled(true);
+	MainWindow::instance()->setEnabledToolbar(true);
+	MainWindow::instance()->setApplicationState(ApplicationState::Default);
+	MapWidget::instance()->repopulateLabels();
+	MultiFilterSortModel::instance()->stopFilterDiveSites();
 }
 
 void LocationInformationWidget::initFields(dive_site *ds)
 {
 	diveSite = ds;
 	if (ds) {
-		copy_taxonomy(&ds->taxonomy, &taxonomy);
 		filter_model.set(ds, ds->location);
 		updateLabels();
 		enableLocationButtons(dive_site_has_gps_location(ds));
-		QSortFilterProxyModel *m = qobject_cast<QSortFilterProxyModel *>(ui.diveSiteListView->model());
-		MultiFilterSortModel::instance()->startFilterDiveSite(ds);
-		if (m)
-			m->invalidate();
+		MultiFilterSortModel::instance()->startFilterDiveSites(QVector<dive_site *>{ ds });
+		filter_model.invalidate();
 	} else {
-		free_taxonomy(&taxonomy);
 		filter_model.set(0, location_t { degrees_t{ 0 }, degrees_t{ 0 } });
 		clearLabels();
 	}
-	MapWidget::instance()->prepareForGetDiveCoordinates(ds);
 }
 
-void LocationInformationWidget::markChangedWidget(QWidget *w)
+void LocationInformationWidget::on_diveSiteCoordinates_editingFinished()
 {
-	QPalette p;
-	qreal h, s, l, a;
-	if (!modified)
-		enableEdition();
-	qApp->palette().color(QPalette::Text).getHslF(&h, &s, &l, &a);
-	p.setBrush(QPalette::Base, (l <= 0.3) ? QColor(Qt::yellow).lighter() : (l <= 0.6) ? QColor(Qt::yellow).light() : /* else */ QColor(Qt::yellow).darker(300));
-	w->setPalette(p);
-	modified = true;
+	if (diveSite)
+		Command::editDiveSiteLocation(diveSite, parseGpsText(ui.diveSiteCoordinates->text()));
 }
 
-void LocationInformationWidget::resetState()
+void LocationInformationWidget::on_diveSiteCountry_editingFinished()
 {
-	modified = false;
-	resetPallete();
-	MainWindow::instance()->diveList->setEnabled(true);
-	MainWindow::instance()->setEnabledToolbar(true);
-	ui.diveSiteMessage->setText(tr("Dive site management"));
-	MapWidget::instance()->endGetDiveCoordinates();
-	MapWidget::instance()->repopulateLabels();
-	MultiFilterSortModel::instance()->stopFilterDiveSite();
-	emit endEditDiveSite();
-	updateLocationOnMap();
+	if (diveSite)
+		Command::editDiveSiteCountry(diveSite, ui.diveSiteCountry->text());
 }
 
-void LocationInformationWidget::enableEdition()
+void LocationInformationWidget::on_diveSiteDescription_editingFinished()
 {
-	MainWindow::instance()->diveList->setEnabled(false);
-	MainWindow::instance()->setEnabledToolbar(false);
-	ui.diveSiteMessage->setText(tr("You are editing a dive site"));
+	if (diveSite)
+		Command::editDiveSiteDescription(diveSite, ui.diveSiteDescription->text());
 }
 
-void LocationInformationWidget::on_diveSiteCoordinates_textChanged(const QString &text)
+void LocationInformationWidget::on_diveSiteName_editingFinished()
 {
-	if (!diveSite)
-		return;
-	location_t location;
-	bool ok_old = has_location(&diveSite->location);
-	bool ok = parseGpsText(text, location);
-	if (ok != ok_old || !same_location(&location, &diveSite->location)) {
-		if (ok) {
-			markChangedWidget(ui.diveSiteCoordinates);
-			enableLocationButtons(true);
-			filter_model.setCoordinates(location);
-		} else {
-			enableLocationButtons(false);
-		}
-	}
+	if (diveSite)
+		Command::editDiveSiteName(diveSite, ui.diveSiteName->text());
 }
 
-void LocationInformationWidget::on_diveSiteCountry_textChanged(const QString& text)
+void LocationInformationWidget::on_diveSiteNotes_editingFinished()
 {
-	if (!same_string(qPrintable(text), taxonomy_get_country(&taxonomy)))
-		markChangedWidget(ui.diveSiteCountry);
+	if (diveSite)
+		Command::editDiveSiteNotes(diveSite, ui.diveSiteNotes->toPlainText());
 }
 
-void LocationInformationWidget::on_diveSiteDescription_textChanged(const QString &text)
+void LocationInformationWidget::on_diveSiteDistance_textChanged(const QString &s)
 {
-	if (diveSite && !same_string(qPrintable(text), diveSite->description))
-		markChangedWidget(ui.diveSiteDescription);
-}
-
-void LocationInformationWidget::on_diveSiteName_textChanged(const QString &text)
-{
-	if (diveSite && !same_string(qPrintable(text), diveSite->name))
-		markChangedWidget(ui.diveSiteName);
-}
-
-void LocationInformationWidget::on_diveSiteNotes_textChanged()
-{
-	if (diveSite && !same_string(qPrintable(ui.diveSiteNotes->toPlainText()), diveSite->notes))
-		markChangedWidget(ui.diveSiteNotes);
-}
-
-void LocationInformationWidget::resetPallete()
-{
-	QPalette p;
-	ui.diveSiteCoordinates->setPalette(p);
-	ui.diveSiteDescription->setPalette(p);
-	ui.diveSiteCountry->setPalette(p);
-	ui.diveSiteName->setPalette(p);
-	ui.diveSiteNotes->setPalette(p);
+	bool ok;
+	uint64_t d = s.toLongLong(&ok);
+	if (!ok)
+		d = 0;
+	closeDistance = prefs.units.length == units::METERS ? d * 1000 : feet_to_mm(d);
+	filter_model.setDistance(closeDistance);
 }
 
 void LocationInformationWidget::reverseGeocode()
 {
-	location_t location;
-	if (!parseGpsText(ui.diveSiteCoordinates->text(), location))
+	location_t location = parseGpsText(ui.diveSiteCoordinates->text());
+	if (!diveSite || !has_location(&location))
 		return;
+	taxonomy_data taxonomy = { 0 };
 	reverseGeoLookup(location.lat, location.lon, &taxonomy);
-	ui.locationTags->setText(constructLocationTags(&taxonomy, false));
+	Command::editDiveSiteTaxonomy(diveSite, taxonomy);
 }
 
-void LocationInformationWidget::updateLocationOnMap()
-{
-	if (!diveSite)
-		return;
-	location_t location;
-	if (!parseGpsText(ui.diveSiteCoordinates->text(), location))
-		return;
-	MapWidget::instance()->updateDiveSiteCoordinates(diveSite, location);
-	filter_model.setCoordinates(location);
-}
-
-DiveLocationFilterProxyModel::DiveLocationFilterProxyModel(QObject*)
+DiveLocationFilterProxyModel::DiveLocationFilterProxyModel(QObject *) : currentLocation({0, 0})
 {
 }
 
-DiveLocationLineEdit *location_line_edit = 0;
+void DiveLocationFilterProxyModel::setFilter(const QString &filterIn)
+{
+	filter = filterIn;
+	invalidate();
+	sort(LocationInformationModel::NAME);
+}
+
+void DiveLocationFilterProxyModel::setCurrentLocation(location_t loc)
+{
+	currentLocation = loc;
+	sort(LocationInformationModel::NAME);
+}
 
 bool DiveLocationFilterProxyModel::filterAcceptsRow(int source_row, const QModelIndex&) const
 {
+	// We don't want to show the first two entries (add dive site with that name)
+	// if there is no filter text.
+	if (filter.isEmpty() && source_row <= 1)
+		return false;
+
 	if (source_row == 0)
 		return true;
 
 	QString sourceString = sourceModel()->index(source_row, LocationInformationModel::NAME).data(Qt::DisplayRole).toString();
-	return sourceString.toLower().contains(location_line_edit->text().toLower());
+	return sourceString.contains(filter, Qt::CaseInsensitive);
 }
 
 bool DiveLocationFilterProxyModel::lessThan(const QModelIndex &source_left, const QModelIndex &source_right) const
 {
-	return source_left.data().toString() < source_right.data().toString();
+	// The first two entries are special - we never want to change their order
+	if (source_left.row() <= 1 || source_right.row() <= 1)
+		return source_left.row() < source_right.row();
+
+	// If there is a current location, sort by that - otherwise use the provided column
+	if (has_location(&currentLocation)) {
+		// The dive sites are -2 because of the first two items.
+		struct dive_site *ds1 = get_dive_site(source_left.row() - 2, &dive_site_table);
+		struct dive_site *ds2 = get_dive_site(source_right.row() - 2, &dive_site_table);
+		return get_distance(&ds1->location, &currentLocation) < get_distance(&ds2->location, &currentLocation);
+	}
+	return source_left.data().toString().compare(source_right.data().toString(), Qt::CaseInsensitive) < 0;
 }
 
-DiveLocationModel::DiveLocationModel(QObject*)
+DiveLocationModel::DiveLocationModel(QObject *)
 {
 	resetModel();
 }
@@ -386,16 +323,17 @@ QVariant DiveLocationModel::data(const QModelIndex &index, int role) const
 		case Qt::DisplayRole:
 			return new_ds_value[index.row()];
 		case Qt::ToolTipRole:
-			return displayed_dive.dive_site ?
+			return current_dive && current_dive->dive_site ?
 				tr("Create a new dive site, copying relevant information from the current dive.") :
 				tr("Create a new dive site with this name");
 		case Qt::DecorationRole:
 			return plusIcon;
 		}
+		return QVariant();
 	}
 
 	// The dive sites are -2 because of the first two items.
-	struct dive_site *ds = get_dive_site(index.row() - 2);
+	struct dive_site *ds = get_dive_site(index.row() - 2, &dive_site_table);
 	return LocationInformationModel::getDiveSiteData(ds, index.column(), role);
 }
 
@@ -426,17 +364,14 @@ DiveLocationLineEdit::DiveLocationLineEdit(QWidget *parent) : QLineEdit(parent),
 							      proxy(new DiveLocationFilterProxyModel()),
 							      model(new DiveLocationModel()),
 							      view(new DiveLocationListView()),
-							      currType(NO_DIVE_SITE),
 							      currDs(nullptr)
 {
-	location_line_edit = this;
-
 	proxy->setSourceModel(model);
 	proxy->setFilterKeyColumn(LocationInformationModel::NAME);
 
 	view->setModel(proxy);
 	view->setModelColumn(LocationInformationModel::NAME);
-	view->setItemDelegate(new LocationFilterDelegate());
+	view->setItemDelegate(&delegate);
 	view->setEditTriggers(QAbstractItemView::NoEditTriggers);
 	view->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 	view->setSelectionBehavior(QAbstractItemView::SelectRows);
@@ -453,7 +388,7 @@ DiveLocationLineEdit::DiveLocationLineEdit(QWidget *parent) : QLineEdit(parent),
 	connect(view, &DiveLocationListView::currentIndexChanged, this, &DiveLocationLineEdit::currentChanged);
 }
 
-bool DiveLocationLineEdit::eventFilter(QObject*, QEvent *e)
+bool DiveLocationLineEdit::eventFilter(QObject *, QEvent *e)
 {
 	if (e->type() == QEvent::KeyPress) {
 		QKeyEvent *keyEv = (QKeyEvent *)e;
@@ -496,9 +431,8 @@ bool DiveLocationLineEdit::eventFilter(QObject*, QEvent *e)
 
 void DiveLocationLineEdit::focusOutEvent(QFocusEvent *ev)
 {
-	if (!view->isVisible()) {
+	if (!view->isVisible())
 		QLineEdit::focusOutEvent(ev);
-	}
 }
 
 void DiveLocationLineEdit::itemActivated(const QModelIndex &index)
@@ -508,13 +442,8 @@ void DiveLocationLineEdit::itemActivated(const QModelIndex &index)
 		idx = index.model()->index(index.row(), LocationInformationModel::NAME);
 
 	dive_site *ds = index.model()->index(index.row(), LocationInformationModel::DIVESITE).data().value<dive_site *>();
-	currType = ds == RECENTLY_ADDED_DIVESITE ? NEW_DIVE_SITE : EXISTING_DIVE_SITE;
 	currDs = ds;
 	setText(idx.data().toString());
-	if (currType == NEW_DIVE_SITE)
-		qDebug() << "Setting a New dive site";
-	else
-		qDebug() << "Setting a Existing dive site";
 	if (view->isVisible())
 		view->hide();
 	emit diveSiteSelected();
@@ -529,7 +458,7 @@ static struct dive_site *get_dive_site_name_start_which_str(const QString &str)
 {
 	struct dive_site *ds;
 	int i;
-	for_each_dive_site (i, ds) {
+	for_each_dive_site (i, ds, &dive_site_table) {
 		QString dsName(ds->name);
 		if (dsName.toLower().startsWith(str.toLower())) {
 			return ds;
@@ -538,7 +467,7 @@ static struct dive_site *get_dive_site_name_start_which_str(const QString &str)
 	return NULL;
 }
 
-void DiveLocationLineEdit::setTemporaryDiveSiteName(const QString&)
+void DiveLocationLineEdit::setTemporaryDiveSiteName(const QString &name)
 {
 	// This function fills the first two entries with potential names of
 	// a dive site to be generated. The first entry is simply the entered
@@ -546,21 +475,21 @@ void DiveLocationLineEdit::setTemporaryDiveSiteName(const QString&)
 	// with the entered text.
 	QModelIndex i0 = model->index(0, LocationInformationModel::NAME);
 	QModelIndex i1 = model->index(1, LocationInformationModel::NAME);
-	model->setData(i0, text());
+	model->setData(i0, name);
 
 	// Note: if i1_name stays empty, the line will automatically
 	// be filtered out by the proxy filter, as it does not contain
 	// the user entered text.
 	QString i1_name;
-	if (struct dive_site *ds = get_dive_site_name_start_which_str(text())) {
+	if (struct dive_site *ds = get_dive_site_name_start_which_str(name)) {
 		const QString orig_name = QString(ds->name).toLower();
-		const QString new_name = text().toLower();
+		const QString new_name = name.toLower();
 		if (new_name != orig_name)
 			i1_name = QString(ds->name);
 	}
 
 	model->setData(i1, i1_name);
-	proxy->invalidate();
+	proxy->setFilter(name);
 	fixPopupPosition();
 	if (!view->isVisible())
 		view->show();
@@ -574,12 +503,10 @@ void DiveLocationLineEdit::keyPressEvent(QKeyEvent *ev)
 	    ev->key() != Qt::Key_Escape &&
 	    ev->key() != Qt::Key_Return) {
 
-		if (ev->key() != Qt::Key_Up && ev->key() != Qt::Key_Down) {
-			currType = NEW_DIVE_SITE;
+		if (ev->key() != Qt::Key_Up && ev->key() != Qt::Key_Down)
 			currDs = RECENTLY_ADDED_DIVESITE;
-		} else {
+		else
 			showPopup();
-		}
 	} else if (ev->key() == Qt::Key_Escape) {
 		view->hide();
 	}
@@ -622,35 +549,44 @@ void DiveLocationLineEdit::fixPopupPosition()
 	}
 }
 
-void DiveLocationLineEdit::setCurrentDiveSite(struct dive_site *ds)
+void DiveLocationLineEdit::setCurrentDiveSite(struct dive *d)
 {
+	struct dive_site *ds = get_dive_site_for_dive(d);
 	currDs = ds;
-	if (!currDs) {
-		currType = NO_DIVE_SITE;
+	if (!currDs)
 		clear();
-	} else {
+	else
 		setText(ds->name);
-	}
+
+	location_t currentLocation = d ? dive_get_gps_location(d) : location_t{0, 0};
+	proxy->setCurrentLocation(currentLocation);
+	delegate.setCurrentLocation(currentLocation);
 }
 
 void DiveLocationLineEdit::showPopup()
 {
-	fixPopupPosition();
-	if (!view->isVisible()) {
+	if (!view->isVisible())
 		setTemporaryDiveSiteName(text());
-		proxy->invalidate();
-		view->show();
-	}
 }
 
-DiveLocationLineEdit::DiveSiteType DiveLocationLineEdit::currDiveSiteType() const
+void DiveLocationLineEdit::showAllSites()
 {
-	return currType;
+	if (!view->isVisible()) {
+		// By setting the "temporary dive site name" to the empty string,
+		// all dive sites are shown sorted by distance from the site of
+		// the current dive.
+		setTemporaryDiveSiteName(QString());
+
+		// By selecting the whole text, the user can immediately start
+		// typing to activate the full-text filter.
+		selectAll();
+	}
 }
 
 struct dive_site *DiveLocationLineEdit::currDiveSite() const
 {
-	return currDs;
+	// If there is no text, this corresponds to the empty dive site
+	return text().trimmed().isEmpty() ? nullptr : currDs;
 }
 
 DiveLocationListView::DiveLocationListView(QWidget*)
