@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "abstractpreferenceswidget.h"
 
-AbstractPreferencesWidget::AbstractPreferencesWidget(const QString& name, const QIcon& icon, float positionHeight)
+AbstractPreferencesWidget::AbstractPreferencesWidget(const QString& name, const QIcon& icon, double positionHeight)
 : QWidget(), _icon(icon), _name(name), _positionHeight(positionHeight)
 {
 }
@@ -16,7 +16,7 @@ QString AbstractPreferencesWidget::name() const
 	return _name;
 }
 
-float AbstractPreferencesWidget::positionHeight() const
+double AbstractPreferencesWidget::positionHeight() const
 {
 	return _positionHeight;
 }
