@@ -20,6 +20,7 @@
 #include "qt-models/divelocationmodel.h"
 #include "qt-models/filtermodels.h"
 #include "core/divesite.h"
+#include "core/errorhelper.h"
 #include "core/subsurface-string.h"
 #include "core/gettextfromc.h"
 #include "desktop-widgets/locationinformation.h"
@@ -44,7 +45,6 @@
 struct Completers {
 	QCompleter *divemaster;
 	QCompleter *buddy;
-	QCompleter *suit;
 	QCompleter *tags;
 };
 
@@ -126,15 +126,12 @@ MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 	Completers completers;
 	completers.buddy = new QCompleter(&buddyModel, ui.buddy);
 	completers.divemaster = new QCompleter(&diveMasterModel, ui.divemaster);
-	completers.suit = new QCompleter(&suitModel, ui.suit);
 	completers.tags = new QCompleter(&tagModel, ui.tagWidget);
 	completers.buddy->setCaseSensitivity(Qt::CaseInsensitive);
 	completers.divemaster->setCaseSensitivity(Qt::CaseInsensitive);
-	completers.suit->setCaseSensitivity(Qt::CaseInsensitive);
 	completers.tags->setCaseSensitivity(Qt::CaseInsensitive);
 	ui.buddy->setCompleter(completers.buddy);
 	ui.divemaster->setCompleter(completers.divemaster);
-	ui.suit->setCompleter(completers.suit);
 	ui.tagWidget->setCompleter(completers.tags);
 	ui.diveNotesMessage->hide();
 	ui.multiDiveWarningMessage->hide();
@@ -308,9 +305,6 @@ void MainTab::divesChanged(const QVector<dive *> &dives, DiveField field)
 	case DiveField::VISIBILITY:
 		ui.visibility->setCurrentStars(current_dive->visibility);
 		break;
-	case DiveField::SUIT:
-		ui.suit->setText(QString(current_dive->suit));
-		break;
 	case DiveField::NOTES:
 		updateNotes(current_dive);
 		break;
@@ -392,16 +386,27 @@ void MainTab::updateMode(struct dive *d)
 	MainWindow::instance()->graphics->recalcCeiling();
 }
 
-void MainTab::updateDateTime(struct dive *d)
+static QDateTime timestampToDateTime(timestamp_t when)
 {
 	// Subsurface always uses "local time" as in "whatever was the local time at the location"
 	// so all time stamps have no time zone information and are in UTC
-	QDateTime localTime = QDateTime::fromMSecsSinceEpoch(1000*d->when, Qt::UTC);
+	QDateTime localTime = QDateTime::fromMSecsSinceEpoch(1000 * when, Qt::UTC);
 	localTime.setTimeSpec(Qt::UTC);
+	return localTime;
+}
+void MainTab::updateDateTime(const struct dive *d)
+{
+	QDateTime localTime = timestampToDateTime(d->when);
 	ui.dateEdit->setDate(localTime.date());
 	ui.timeEdit->setTime(localTime.time());
 }
 
+void MainTab::updateTripDate(const struct dive_trip *t)
+{
+	QDateTime localTime = timestampToDateTime(trip_date(t));
+	ui.dateEdit->setDate(localTime.date());
+}
+
 void MainTab::updateDiveSite(struct dive *d)
 {
 	struct dive_site *ds = d->dive_site;
@@ -463,8 +468,6 @@ void MainTab::updateDiveInfo()
 			ui.DivemasterLabel->setVisible(false);
 			ui.buddy->setVisible(false);
 			ui.BuddyLabel->setVisible(false);
-			ui.suit->setVisible(false);
-			ui.SuitLabel->setVisible(false);
 			ui.rating->setVisible(false);
 			ui.RatingLabel->setVisible(false);
 			ui.visibility->setVisible(false);
@@ -487,6 +490,7 @@ void MainTab::updateDiveInfo()
 			// rename the remaining fields and fill data from selected trip
 			ui.LocationLabel->setText(tr("Trip location"));
 			ui.diveTripLocation->setText(currentTrip->location);
+			updateTripDate(currentTrip);
 			ui.locationTags->clear();
 			//TODO: Fix this.
 			//ui.location->setText(currentTrip->location);
@@ -518,8 +522,6 @@ void MainTab::updateDiveInfo()
 			ui.editDiveSiteButton->show();
 			ui.divemaster->setVisible(true);
 			ui.buddy->setVisible(true);
-			ui.suit->setVisible(true);
-			ui.SuitLabel->setVisible(true);
 			ui.rating->setVisible(true);
 			ui.RatingLabel->setVisible(true);
 			ui.visibility->setVisible(true);
@@ -554,7 +556,6 @@ void MainTab::updateDiveInfo()
 			updateMode(current_dive);
 			updateDiveSite(current_dive);
 			updateDateTime(current_dive);
-			ui.suit->setText(current_dive->suit);
 			ui.divemaster->setText(current_dive->divemaster);
 			ui.buddy->setText(current_dive->buddy);
 			ui.airtemp->setText(get_temperature_string(current_dive->airtemp, true));
@@ -577,7 +578,6 @@ void MainTab::updateDiveInfo()
 		ui.rating->setCurrentStars(0);
 		ui.visibility->setCurrentStars(0);
 		ui.location->clear();
-		ui.suit->clear();
 		ui.divemaster->clear();
 		ui.buddy->clear();
 		ui.airtemp->clear();
@@ -600,7 +600,6 @@ void MainTab::updateDiveInfo()
 
 void MainTab::reload()
 {
-	suitModel.updateModel();
 	buddyModel.updateModel();
 	diveMasterModel.updateModel();
 	tagModel.updateModel();
@@ -877,14 +876,6 @@ void MainTab::on_diveTripLocation_editingFinished()
 	Command::editTripLocation(currentTrip, ui.diveTripLocation->text());
 }
 
-void MainTab::on_suit_editingFinished()
-{
-	if (editMode == IGNORE || !current_dive)
-		return;
-
-	divesEdited(Command::editSuit(ui.suit->text(), false));
-}
-
 void MainTab::on_notes_editingFinished()
 {
 	if (!currentTrip && !current_dive)
