@@ -52,7 +52,8 @@ slots:
 	void updateDiveInfo();
 	void updateNotes(const struct dive *d);
 	void updateMode(struct dive *d);
-	void updateDateTime(struct dive *d);
+	void updateDateTime(const struct dive *d);
+	void updateTripDate(const struct dive_trip *t);
 	void updateDiveSite(struct dive *d);
 	void acceptChanges();
 	void rejectChanges();
@@ -60,7 +61,6 @@ slots:
 	void on_locationPopupButton_clicked();
 	void on_divemaster_editingFinished();
 	void on_buddy_editingFinished();
-	void on_suit_editingFinished();
 	void on_diveTripLocation_editingFinished();
 	void on_notes_editingFinished();
 	void on_airtemp_editingFinished();
@@ -85,7 +85,6 @@ private:
 	EditMode editMode;
 	BuddyCompletionModel buddyModel;
 	DiveMasterCompletionModel diveMasterModel;
-	SuitCompletionModel suitModel;
 	TagCompletionModel tagModel;
 	bool modified;
 	bool lastSelectedDive;
