@@ -59,17 +59,13 @@ while [[ $# -gt 0 ]] ; do
 			# we are building an AppImage as by product
 			CREATE_APPDIR="1"
 			;;
-		-skip-googlemaps)
-			# hack for Travix Mac build
-			SKIP_GOOGLEMAPS="1"
-			;;
 		-release)
 			# don't build Debug binaries
 			DEBUGRELEASE="Release"
 			;;
 		*)
 			echo "Unknown command line argument $arg"
-			echo "Usage: build.sh [-no-bt] [-build-deps] [-build-with-webkit] [-mobile] [-desktop] [-both] [-create-appdir] [-skip-googlemaps] [-release]"
+			echo "Usage: build.sh [-no-bt] [-build-deps] [-build-with-webkit] [-mobile] [-desktop] [-both] [-create-appdir] [-release]"
 			exit 1
 			;;
 	esac
@@ -312,12 +308,6 @@ if [[ $PLATFORM = Darwin && "$BUILD_DEPS" == "1" ]] ; then
 	make -j4 -k
 	make -k install
 	popd
-else
-	# we are getting libusb and hidapi from pkg-config and that goes wrong
-	# or more specifically, the way libdivecomputer references
-	# the include files goes wrong
-	pkg-config --exists libusb-1.0 && LIBDC_CFLAGS=-I$(dirname $(pkg-config --cflags libusb-1.0 | sed -e 's/^-I//'))
-	pkg-config --exists hidapi && LIBDC_CFLAGS="${LIBDC_CFLAGS} -I$(dirname $(pkg-config --cflags hidapi | sed -e 's/^-I//'))"
 fi
 
 
@@ -421,27 +411,25 @@ else
 	PRINTING="-DNO_PRINTING=ON"
 fi
 
-if [ "$SKIP_GOOGLEMAPS" != "1" ] ; then
-	# build the googlemaps map plugin
+# build the googlemaps map plugin
 
-	cd $SRC
-	./subsurface/scripts/get-dep-lib.sh single . googlemaps
-	pushd googlemaps
-	mkdir -p build
-	mkdir -p J10build
-	cd build
-	$QMAKE "INCLUDEPATH=$INSTALL_ROOT/include" ../googlemaps.pro
-	# on Travis the compiler doesn't support c++1z, yet qmake adds that flag;
-	# since things compile fine with c++11, let's just hack that away
-	# similarly, don't use -Wdata-time
-	if [ "$TRAVIS" = "true" ] ; then
-		mv Makefile Makefile.bak
-		cat Makefile.bak | sed -e 's/std=c++1z/std=c++11/g ; s/-Wdate-time//' > Makefile
-	fi
-	make -j4
-	make install
-	popd
+cd $SRC
+./subsurface/scripts/get-dep-lib.sh single . googlemaps
+pushd googlemaps
+mkdir -p build
+mkdir -p J10build
+cd build
+$QMAKE "INCLUDEPATH=$INSTALL_ROOT/include" ../googlemaps.pro
+# on Travis the compiler doesn't support c++1z, yet qmake adds that flag;
+# since things compile fine with c++11, let's just hack that away
+# similarly, don't use -Wdata-time
+if [ "$TRAVIS" = "true" ] ; then
+	mv Makefile Makefile.bak
+	cat Makefile.bak | sed -e 's/std=c++1z/std=c++11/g ; s/-Wdate-time//' > Makefile
 fi
+make -j4
+make install
+popd
 
 # finally, build Subsurface
 
