@@ -1,7 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0
 #include <QQmlEngine>
-#include <QDesktopWidget>
-#include <QApplication>
 #include <QDebug>
 #include <QQuickItem>
 
@@ -55,13 +53,6 @@ void exit_ui()
 	free((void *)existing_filename);
 }
 
-double get_screen_dpi()
-{
-	QDesktopWidget *mydesk = qApp->desktop();
-	return mydesk->physicalDpiX();
-}
-
-
 void run_ui()
 {
 
@@ -76,8 +67,8 @@ void run_ui()
 	// same directory the executable was started from <bundle>/Contents/MacOS/
 	// To work around this we need to manually copy the components at install time
 	// to Contents/Frameworks/qml and make sure that we add the correct import path
-	QStringList importPathList = engine.importPathList();
-	Q_FOREACH (QString importPath, importPathList) {
+	const QStringList importPathList = engine.importPathList();
+	for (QString importPath: importPathList) {
 		if (importPath.contains("MacOS"))
 			engine.addImportPath(importPath.replace("MacOS", "Frameworks"));
 	}
@@ -158,7 +149,7 @@ static void register_meta_types()
 void register_qml_types(QQmlEngine *engine)
 {
 	// register qPref*
-	qPref::instance()->registerQML(engine);
+	qPref::registerQML(engine);
 
 #ifndef SUBSURFACE_TEST_DATA
 	int rc;
